/*++

(C) Copyright 2000-2005 Hewlett-Packard Development Company, L.P.

Module Name:

    CpqCiDrv.h

Abstract:

    This header file contains the prototypes and data types 
    required to support the channel driver entrypoints.

Environment:

    OS device driver context.

Notes:

Revision History:

--*/

#ifndef _CPQCIDRVh
#define _CPQCIDRVh

#include "CpqCi.h"

/*****************************************************************************
    The channel object is used to track channel kernel resources.
    *** Note that "DriverCcb" field is also used as the creation key.
******************************************************************************/

typedef struct _CPQCICHANNELOBJECT {
    CPQCITYPEQUAD ReferenceCount;  // # of processes connected to this channel
    CPQCITYPEQUAD WaiterCount;     // # of waiters registered on receive Q
    CPQCITYPEQUAD SqFifoOffset;    // relative offset from MemoryVaAdj
    CPQCITYPEQUAD SqDescOffset;    // relative offset from MemoryVaAdj
    CPQCITYPEQUAD RqFifoOffset;    // relative offset from MemoryVaAdj
    CPQCITYPEQUAD RqDescOffset;    // relative offset from MemoryVaAdj
    CPQCITYPEQUAD DbBaseVaAdj;     // this channel's base doorbell page kVA
    CPQCITYPEQUAD DbBasePaAdj;     // this channel's base doorbell page PA
    CPQCITYPEQUAD MemoryVaAdj;     // adjusted usable memory kVA
    CPQCITYPEQUAD MemoryPaAdj;     // adjusted usable memory PA
    CPQCISIZE     MemorySizeAdj;   // adjusted usable memory size
    CPQCIHANDLE   MemoryHandle;    // handle to allocated channel memory
    CPQCIFIFOBAR  QPriority;       // target channel's priority Q
    CPQCICCB      DriverCcb;       // driver reference to ccb for channel
    CPQCIKEY      Key;             // driver derived key to track chnl access
} CPQCICHANNELOBJECT;

/*****************************************************************************
    The connection object is used to track a connection's resources mapped
    into the caller's address space (usermode/kernelmode).
******************************************************************************/

typedef struct _CPQCICONNECTIONOBJECT {
    CPQCIHANDLE  MappedMemoryHandle;   // mapped handle of channel MemoryPaAdj
    CPQCIHANDLE  MappedDoorbellHandle; // mapped handle of channel DbBasePa
    CPQCICCB     MappedCcbCopy;        // copy of the caller's ccb context
    CPQCIHANDLE  Context;  	       // context in which connection lives (file* for Linux). 
} CPQCICONNECTIONOBJECT;

/*****************************************************************************
    The adapter object is used to track adapter kernel resources.
******************************************************************************/

typedef struct _CPQCIADAPTEROBJECT {
    CPQCICONNECTIONS MaxConnections;   // max connections supported by adapter
    CPQCICHANNELS MaxChannels;         // max channels supported by adapter
    CPQCIHANDLE   AdapterObjMemHandle; // handle to kallocated adapter obj
    CPQCIHANDLE   MmioRegistersHandle; // handle to kmapped mmio registers
    CPQCIHANDLE   MmioCcbTableHandle;  // handle to kmapped mmio ccb table
    CPQCIHANDLE   MmioDoorbellHandle;  // handle to kmapped mmio doorbell pgs
    CPQCIFIFOBAR  QConnectionObjs;     // cnb connection object fifo Q
    CPQCIFIFOBAR  QNormPriority;       // cnb normal priority channel fifo Q
    CPQCIFIFOBAR  QHighPriority;       // cnb high priority fifo Q
    CPQCIFIFOBAR  QCri0Priority;       // cnb critical priority 0 fifo Q
    CPQCIFIFOBAR  QCri1Priority;       // cnb critical priority 1 fifo Q
    CPQCIFIFOBAR  QCri2Priority;       // cnb critical priority 2 fifo Q
    CPQCIFIFOBAR  QCri3Priority;       // cnb critical priority 3 fifo Q
    CPQCIFIFOBAR  QCri4Priority;       // cnb critical priority 4 fifo Q
    CPQCIFIFOBAR  QCri5Priority;       // cnb critical priority 5 fifo Q
    CPQCIFIFOBAR  QCri6Priority;       // cnb critical priority 6 fifo Q
    CPQCIFIFOBAR  QCri7Priority;       // cnb critical priority 7 fifo Q
    CPQCICHANNELOBJECT *ChannelObject;
    CPQCICONNECTIONOBJECT *ConnectionObject;
} CPQCIADAPTEROBJECT, *CPQCIADAPTER;

/*****************************************************************************
    Prototypes for OS-independent routines found in CPQCIDRV.C.
******************************************************************************/
#ifndef _SKIP_PRIV_PROTO
CPQCISTATUS
CpqCiDrvCreateAdapter(
    CPQCICONTEXT Context,
    CPQCIPHYSICAL RegisterBarPa,
    CPQCISIZE RegisterBarSize,
    CPQCICONNECTIONS MaxConnections,
    CPQCIADAPTER *pAdapter
    );

CPQCISTATUS
CpqCiDrvDeleteAdapter(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter
    );

CPQCISTATUS
CpqCiDrvCreateChannel(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCIENTRIES SendPackets,
    CPQCISIZE SendPacketSize,
    CPQCIENTRIES RecvPackets,
    CPQCISIZE RecvPacketSize,
    CPQCICCB *CallersCcb,
    CPQCIPRIORITY ChannelPriority,
    CPQCIMODE AddressMode,
    CPQCIKEY *pKey,
    CPQCICONNECTION *pConnection,
    CPQCICHANNEL *pChannel
    );

CPQCISTATUS
CpqCiDrvOpenChannel(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCIMODE AddressMode,
    CPQCICCB *CallersCcb,
    CPQCIKEY *pKey,
    CPQCICONNECTION *pConnection,
    CPQCICHANNEL *pChannel
    );

CPQCISTATUS
CpqCiDrvCloseChannel(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCICONNECTION Connection,
    CPQCICHANNEL *pChannel
    );

CPQCISTATUS
CpqCiDrvRegisterWaiter(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCICONNECTION Connection
    );

CPQCISTATUS
CpqCiDrvUnregisterWaiter(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCICHANNEL Channel
    );

CPQCISTATUS
CpqCiDrvGetPendingChannels(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCICHANNEL *pChannel,
    CPQCICHANNELS *pChannelList,
    CPQCICHANNELS *pDeferredList
    );

CPQCILENGTH
CpqCiDrvSend(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCICONNECTION Connection,
    CPQCIBUFFER Buffer,
    CPQCILENGTH Length,
    CPQCISTATUS *pError
    );

CPQCILENGTH
CpqCiDrvRecv(
    CPQCICONTEXT Context,
    CPQCIADAPTER Adapter,
    CPQCICONNECTION Connection,
    CPQCIBUFFER Buffer,
    CPQCILENGTH Length,
    CPQCISTATUS *pError
    );

int
CpqCiShareContext(
	CPQCICONTEXT Context,
	struct _CPQCIADAPTEROBJECT *pao,
	CPQCICONNECTION connection
	);

CPQCIDRVOSHANDLE
GetMemoryHandle(
	struct _CPQCIADAPTEROBJECT *pao,
	CPQCICONNECTION connection
	);
#endif

/*****************************************************************************
    Prototypes for OS-dependent routines found in CPQCIKos.C.
******************************************************************************/

#ifndef _SKIP_PUB_PROTO
CPQCIHANDLE
CpqCiDrvOsMallocContiguous(
    CPQCICONTEXT Context,
    CPQCISIZE AllocationSize
    );

CPQCIHANDLE
CpqCiDrvOsMalloc(
    CPQCICONTEXT Context,
    CPQCISIZE AllocationSize
    );


CPQCISTATUS
CpqCiOsGetRefCount(
	CPQCICONTEXT Context,
	CPQCITYPEQUAD*	RefCount
   );

CPQCIHANDLE
CpqCiDrvOsMap(
    CPQCICONTEXT Context,
    CPQCIPHYSICAL PhysicalAddress,
    CPQCISIZE MappedSize,
    CPQCIMODE AddressMode,
    CPQCICACHETYPE CacheType
    );

CPQCIVOID
CpqCiDrvOsFree(
    CPQCICONTEXT Context,
    CPQCIHANDLE MemoryHandle
    );

CPQCIVIRTUAL
CpqCiDrvOsGetVA(
    CPQCICONTEXT Context,
    CPQCIHANDLE MemoryHandle
    );

CPQCIPHYSICAL
CpqCiDrvOsGetPA(
    CPQCICONTEXT Context,
    CPQCIHANDLE MemoryHandle
    );

CPQCISIZE
CpqCiDrvOsGetSize(
    CPQCICONTEXT Context,
    CPQCIHANDLE MemoryHandle
    );

CPQCICACHETYPE
CpqCiDrvOsGetCacheType(
    CPQCICONTEXT Context,
    CPQCIHANDLE MemoryHandle
    );

CPQCIMODE
CpqCiDrvOsGetMode(
    CPQCICONTEXT Context,
    CPQCIHANDLE MemoryHandle
    );

CPQCIVOID
CpqCiDrvOsStall(
    CPQCICONTEXT Context,
    CPQCITIMEOUT MicroSeconds
    );

CPQCIVIRTUAL CpqCiMemSet( 
	CPQCIVIRTUAL dest, 
	u8_t t, CPQCISIZE s
	);

CPQCISIZE CpqCiSNPrintf(
	CPQCICHAR* buf, 
	CPQCISIZE size, 
	const CPQCICHAR* fmt, 
	...
	);

CPQCISIZE CpqCiSPrintf(
	CPQCICHAR* buf, 
	const CPQCICHAR* fmt, 
	...
	);
int print_mem_handle(
	char* buffer,
	int bufsize,
	CPQCIDRVOSMEMHANDLE handle
	);
#endif
#endif // _CPQCIDRVh
